/***********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS
 * SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
 **********************************************************************************************************************/
/***********************************************************************************************************************
 * File Name    : r_scifa_drv_sc_cfg.h
 * Version      : 1.0
 * Description  : OS timer driver configuration header, part of the high layer driver.
 *                This file uses the same name for any low layer driver that is supported by a compatible high layer
 *                driver.
 *                File is included in the high layer driver to allow the low layer driver to be swapped at build time
 *                and share the same high layer code.
 **********************************************************************************************************************/

#ifndef SRC_RENESAS_CONFIG_R_SCIFA_DRV_SC_CFG_H_
#define SRC_RENESAS_CONFIG_R_SCIFA_DRV_SC_CFG_H_
/******************************************************************************************************************//**
 * @ingroup RENESAS_DRIVER_LIBRARY
 * @defgroup RENESAS_SC_CFG Renesas Smart Configuration Settings
 *
 * @section RENESAS_SC_CFG_SUMMARY Summary
 * This configuration file determines the driver configuration making specific
 * settings for the peripheral.
 *
 * This file is normally generated by the Renesas Smart Configurator tool. It is
 * it is possible to generate this file for different configurations, and replace
 * the file in different builds to change the system configuration.
 *
 * Do not edit this file if using SC
 *
 * @see RENESAS_OS_ABSTRACTION  Renesas OS Abstraction interface
 * @{
 *********************************************************************************************************************/
#include "r_typedefs.h"
#include "r_scifa_drv_link.h"
#include "r_gpio_drv_api.h"
#include "r_gpio_drv_sc_cfg.h"

/**
 * @typedef st_r_drv_scifa_sc_config_t Smart Configurator Interface
 */
typedef struct
{
    int_t                           channel;
    scifa_config_t                  config;
    st_r_drv_gpio_pin_init_table_t  pin;
} st_r_drv_scifa_sc_config_t;

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Variable Externs
******************************************************************************/

/**
 * @section R_SCIFA_SC_CFG_80_TABLE Smart Configurator settings table.
 */
static const st_r_drv_scifa_sc_config_t SCIFA_SC_TABLE[] =
{
    { 4, 
        {
            SCIFA_MODE_ASYNC, 
			115200,
			SCIFA_CLK_SRC_INT_SCK_IN,
            SCIFA_CLK_16X,
            SCIFA_DATA_8BIT, 
            SCIFA_PARITY_OFF, 
            SCIFA_EVEN_PARITY, 
            SCIFA_STOPBITS_1, 
            SCIFA_NOISE_CANCEL_DISABLE, 
            SCIFA_LSB_FIRST, 
            SCIFA_LOOPBACK_DISABLE, 
            SCIFA_MODEM_CONTROL_DISABLE, 
            SCIFA_RTS_ACTIVE_TRIGGER_15, 
            15, 
            1, 
            SCIFA_SPTR_INIT_HIGH, 
            SCIFA_SPTR_INIT_HIGH, 
            SCIFA_SPTR_INIT_HIGH, 
            SCIFA_SPTR_INIT_HIGH, 
            SCIFA_TX_INTERRUPT_MODE, 
            SCIFA_RX_INTERRUPT_MODE, 
            0, 
            30, 
            30, 
            0, 
        }, 
        {
            &GPIO_SC_TABLE_scifa4[0], 
            sizeof(GPIO_SC_TABLE_scifa4)/sizeof(st_r_drv_gpio_sc_config_t), 
        }
    },
    /* End of modification */
};


#endif /* SRC_RENESAS_CONFIG_R_SCIFA_DRV_SC_CFG_H_ */

/*******************************************************************************************************************//**
End  Of File
 ***********************************************************************************************************************/
