/**
 * @file    m13_rza2m_ek_lcd_settings.h
 * @brief   LCD and TCON signals settings
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RZA2M_EK_LCD_SETTINGS_H
#define _M13_RZA2M_EK_LCD_SETTINGS_H

// Dependencies
#include "r_vdc.h"

// Macro

// #define LOGO_16_BIT_PER_PIXEL
#define LOGO_24_BIT_PER_PIXEL

// Select the TFT to drive
#define  TFT_EASTRISING_ER_TFT043_7          (1)

    #ifdef TFT_EASTRISING_ER_TFT043_7
        #define     LCD_CH0_DISP_HS          (20u + 26u)                                     // LCD display area size, horizontal start position
        #define     LCD_CH0_DISP_HW          (800u)                                          // LCD display area size, Width
        #define     LCD_DISP_W_HSYNC         (48u)                                           // Horizontal Pulse Width
        #define     LCD_DISP_W_VSYNC         (3u)                                            // Vertical Pulse Width
        #define     LCD_CH0_DISP_VS          (32u)                                           // LCD display area size, vertical start position
        #define     LCD_CH0_DISP_VW          (480u)                                          // LCD display area size, Height
        #define     LCD_CH0_SIG_FV           (525u)                                          // Vsync period
        #define     LCD_CH0_SIG_FH           (928u)                                          // Hsync period
        #define     LCD_CH0_TCON_HALF        (464u)                                          // TCON reference timing, 1/2 FH timing (Hsync period)
        #define     LCD_CH0_TCON_OFFSET      (0u)                                            // TCON reference timing, offset Hsync signal timing
        #define     LCD_CH0_PANEL_CLK        (VDC_PANEL_ICKSEL_PERI)                         // Panel clock select
        #define     LCD_CH0_PANEL_CLK_DIV    (VDC_PANEL_CLKDIV_1_2)                          // Panel clock frequency division ratio
        #define     LCD_CH0_OUT_EDGE         (VDC_EDGE_RISING)                               // Output phase control of LCD_DATA[23:0] signal
    #endif

    #ifdef LOGO_16_BIT_PER_PIXEL
        #define     LCD_CH0_OUT_FORMAT       (VDC_LCD_OUTFORMAT_RGB565)                      // LCD output format select
    #endif

   #ifdef LOGO_24_BIT_PER_PIXEL
        #define     LCD_CH0_OUT_FORMAT       (VDC_LCD_OUTFORMAT_RGB888)                      // LCD output format select
    #endif

// VDC5_CHANNEL_0 or VDC5_CHANNEL_1
    #define       GRAPHICS_CHANNEL        (VDC_CHANNEL_0)

// Graphics format: RGB565, 2 bytes per pixel
    #ifdef LOGO_16_BIT_PER_PIXEL
        #define     GRAPHICS_FORMAT             (VDC_GR_FORMAT_RGB565)
    #endif
    #ifdef LOGO_24_BIT_PER_PIXEL
        #define     GRAPHICS_FORMAT             (VDC_GR_FORMAT_ARGB888)
    #endif
// Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128 in accordance with the frame buffer burst transfer mode
    #ifdef LOGO_16_BIT_PER_PIXEL
        #define     FRAME_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 2u) + 31u) & ~31u)
    #endif
    #ifdef LOGO_24_BIT_PER_PIXEL
        #define     FRAME_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 4u) + 31u) & ~31u)
    #endif

    #define     FRAME_BUFFER_HEIGHT          (LCD_CH0_DISP_VW)
    #define     VSYNC_WAIT_COUNT             (2)


// BSP Functions
void bsp_set_lcd_panel_ch0(void);
void bsp_set_lcd_tcon_settings(vdc_channel_t channel, const vdc_lcd_tcon_timing_t **outctrl);


#endif
