/**
 * @file m13_rza2m_ek_lcd.h
 * @brief M13-RZ/A2M-EK lcd BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RZA2M_EK_LCD_H
#define _M13_RZA2M_EK_LCD_H

// Dependencies
#include "fonts.h"
#include "m13_rza2m_ek_pinout.h"
#include "m13_rza2m_ek_status.h"
#include "m13_rza2m_ek_config.h"
#include "r_vdc.h"

st_r_drv_gpio_pin_list_t lcd_pin;

typedef uint16_t bsp_color_t;

// User setting
#ifndef BSP_LCD_MAX_FRAME_BUFFER
#define BSP_LCD_MAX_FRAME_BUFFER        1
#endif

#ifndef BSP_LCD_RAM_SECTION
#define BSP_LCD_RAM_SECTION             "Graphics_RAM"
#endif

// LCD display dimensions
#define LCD_WIDTH                      800
#define LCD_HEIGHT                     480
#define LCD_STRIDE                     (((LCD_WIDTH * 2) + 31u) & ~31u)


#define BACKLIGHT_OFF                  GPIO_LEVEL_LOW
#define BACKLIGHT_ON                   GPIO_LEVEL_HIGH
#define DISPLAY_BITS_PER_PIXEL_INPUT0  (16)
typedef enum
{
    ROTATION_ZERO  = 0,
    ROTATION_090   = 90,
    ROTATION_180   = 180,
    ROTATION_270   = 270,
} screen_rotation_t;

// LCD Color codes in BGR565

#define LCD_COLOR_RED                   0x001F
#define LCD_COLOR_GREEN                 0x07E0
#define LCD_COLOR_BLUE                  0xF800

#define LCD_COLOR_RED1                  0x0010
#define LCD_COLOR_GREEN1                0x0400
#define LCD_COLOR_BLUE1                 0x8000

#define LCD_COLOR_CYAN                  0xFFE0
#define LCD_COLOR_MAGENTA               0xF81F
#define LCD_COLOR_YELLOW                0x07FF

#define LCD_COLOR_BLACK                 0x0000
#define LCD_COLOR_WHITE                 0xFFFF
#define LCD_COLOR_M13_GREEN1            0x6509
#define LCD_COLOR_M13_GREEN2            0xBEAA
#define LCD_COLOR_M13_BLUE              0x51E5

#define LCD_DEFAULT_BACKGROUND_COLOR    LCD_COLOR_M13_GREEN1
#define LCD_DEFAULT_TEXT_COLOR          LCD_COLOR_BLACK

#define LCD_DEFAULT_BACKGROUND_COLOR    LCD_COLOR_M13_GREEN1
#define LCD_DEFAULT_TEXT_COLOR          LCD_COLOR_BLACK

// Back light pin definition
#define LCD_BACKLIGHT_PIN               BSP_IO_PORT_00_PIN_15
#define LCD_BACKLIGHT_OFF               BSP_IO_LEVEL_LOW
#define LCD_BACKLIGHT_ON                BSP_IO_LEVEL_HIGH

// Typedef definitions
typedef struct
{
    void *pb1;
    void *pb2;
    void *pcurrent;
} vdc_frame_buffer_t;

// BSP Functions
bsp_status_t bsp_lcd_io_init(void);
void bsp_lcd_init(void);
void bsp_lcd_deinit(void);
void bsp_lcd_config_interrupt(vdc_channel_t ch);

void bsp_lcd_enable_double_buffer(void);
void bsp_lcd_swap_buffer(void);


void bsp_lcd_set_display_buffer(uint8_t index);
void bsp_lcd_set_working_buffer(uint8_t index);

void bsp_lcd_display_on(void);
void bsp_lcd_display_off(void);
void bsp_lcd_clear(bsp_color_t color);
void bsp_lcd_backlight_on(void);
void bsp_lcd_backlight_off(void);

bsp_color_t bsp_lcd_read_pixel(uint16_t x, uint16_t y);
void bsp_lcd_draw_pixel(uint16_t x, uint16_t y, bsp_color_t color);
void bsp_lcd_draw_horizontal_line(uint16_t x, uint16_t y, uint16_t length, bsp_color_t color);
void bsp_lcd_draw_vertical_line(uint16_t x, uint16_t y, uint16_t length, bsp_color_t color);
void bsp_lcd_draw_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color);
void bsp_lcd_fill_rect(uint16_t x, uint16_t y, uint16_t width, uint16_t height, bsp_color_t color);

void bsp_lcd_set_text_background_color(bsp_color_t color);
void bsp_lcd_set_text_color(bsp_color_t color);
void bsp_lcd_set_font(font_t *p_font);
void bsp_lcd_display_char(uint16_t x, uint16_t y, uint8_t c);
void bsp_lcd_display_string(uint16_t x, uint16_t y, const char *s);

#endif

