/**
 * @file m13_rza2m_ek_led.c
 * @brief M13-RZ/A2M-EK led BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "m13_rza2m_ek_led.h"

// Private global variables and functions
static int_t gpio_handle, err;

static st_r_drv_gpio_pin_rw_t user_led1_hi = { GPIO_PORT_J_PIN_1, GPIO_LEVEL_HIGH, GPIO_SUCCESS };
static st_r_drv_gpio_pin_rw_t user_led1_lo = { GPIO_PORT_J_PIN_1, GPIO_LEVEL_LOW, GPIO_SUCCESS };

/**
 * @name       bsp_led_io_init
 * @brief      Configure M13-RZA2M-EK led pin
 * @param[in]  None
 * @return     None
 */
bsp_status_t bsp_led_init(void)
{
    gpio_handle = direct_open("gpio", 0);
    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_led_pin_table);

    if(err < 0)
    {
		return BSP_STATUS_PIN_CONFIG_FAILURE;
    }
	
	return BSP_STATUS_OK;
}

/**
 * @name       bsp_led_on
 * @brief      Turn on led
 * @param[in]  Led pin ID
 * @return     None
 */
void bsp_led_on(r_gpio_port_pin_t pin)
{
    if(pin == LED1)
    {
        direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, &user_led1_hi);
    }
}

/**
 * @name       bsp_led_off
 * @brief      Turn off led
 * @param[in]  Led pin ID
 * @return     None
 */
void bsp_led_off(r_gpio_port_pin_t pin)
{
    if(pin == LED1)
    {
        direct_control(gpio_handle, CTL_GPIO_PIN_WRITE, &user_led1_lo);
    }
}

/**
 * @name       bsp_led_get_state
 * @brief      Read led pin status en return its level
 * @param[in]  Led pin
 * @return     Return pin level
 */
e_r_drv_gpio_level_t bsp_led_get_state(r_gpio_port_pin_t pin)
{
    e_r_drv_gpio_level_t led_level;

    R_GPIO_PinRead(pin, &led_level);

    return led_level;
}

