/**
 * @file m13_rza2m_ek_octaram.h
 * @brief M13-RZ/A2M-EK OctaRAM BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RZA2M_EK_OCTARAM_H
#define _M13_RZA2M_EK_OCTARAM_H

// Macro definitions
#define BSP_OCTARAM_BASE_ADDR_NO_CACHE      0x60000000
#define BSP_OCTARAM_SIZE                    (8 * 1024 * 1024)

// DCR Register Bit position
#define BSP_OCTARAM_DCR_POS_DVCMD1          8
#define BSP_OCTARAM_DCR_POS_DVCMD0          0

// DCR Register Bit Mask
#define BSP_OCTARAM_DCSR_MASK_DVCMD1        (255 << BSP_OCTARAM_DCR_POS_DVCMD1)
#define BSP_OCTARAM_DCSR_MASK_DVCMD0        (255 << BSP_OCTARAM_DCR_POS_DVCMD0)

// DCR Register configuration
#define BSP_OCTARAM_DCSR_DVCMD1(n)          (((n) << BSP_OCTARAM_DCR_POS_DVCMD1) & BSP_OCTARAM_DCSR_MASK_DVCMD1)
#define BSP_OCTARAM_DCSR_DVCMD0(n)          (((n) << BSP_OCTARAM_DCR_POS_DVCMD0) & BSP_OCTARAM_DCSR_MASK_DVCMD0)

// DAR Register Bit position
#define BSP_OCTARAM_DAR_POS_DVAD3           24
#define BSP_OCTARAM_DAR_POS_DVAD2           16
#define BSP_OCTARAM_DAR_POS_DVAD1           8
#define BSP_OCTARAM_DAR_POS_DVAD0           0

// DAR Register Bit mask
#define BSP_OCTARAM_DAR_MASK_DVAD3          (255 << BSP_OCTARAM_DAR_POS_DVAD3)
#define BSP_OCTARAM_DAR_MASK_DVAD2          (255 << BSP_OCTARAM_DAR_POS_DVAD2)
#define BSP_OCTARAM_DAR_MASK_DVAD1          (255 << BSP_OCTARAM_DAR_POS_DVAD1)
#define BSP_OCTARAM_DAR_MASK_DVAD0          (255 << BSP_OCTARAM_DAR_POS_DVAD0)

// DAR Register configuration
#define BSP_OCTARAM_DAR_DVAD3(n)            (((n) << BSP_OCTARAM_DAR_POS_DVAD3) & BSP_OCTARAM_DAR_MASK_DVAD3)
#define BSP_OCTARAM_DAR_DVAD2(n)            (((n) << BSP_OCTARAM_DAR_POS_DVAD2) & BSP_OCTARAM_DAR_MASK_DVAD2)
#define BSP_OCTARAM_DAR_DVAD1(n)            (((n) << BSP_OCTARAM_DAR_POS_DVAD1) & BSP_OCTARAM_DAR_MASK_DVAD1)
#define BSP_OCTARAM_DAR_DVAD0(n)            (((n) << BSP_OCTARAM_DAR_POS_DVAD0) & BSP_OCTARAM_DAR_MASK_DVAD0)

// DCSR Register Bit position
#define BSP_OCTARAM_DCSR_POS_ACDA           28
#define BSP_OCTARAM_DCSR_POS_DOPI           27
#define BSP_OCTARAM_DCSR_POS_ADLEN          24
#define BSP_OCTARAM_DCSR_POS_DAOR           23
#define BSP_OCTARAM_DCSR_POS_CMDLEN         20
#define BSP_OCTARAM_DCSR_POS_ACDV           19
#define BSP_OCTARAM_DCSR_POS_DMLEN          8
#define BSP_OCTARAM_DCSR_POS_DALEN          0

// DCSR Register Bit Mask
#define BSP_OCTARAM_DCSR_MASK_ACDA          (1   << BSP_OCTARAM_DCSR_POS_ACDA)
#define BSP_OCTARAM_DCSR_MASK_DOPI          (1   << BSP_OCTARAM_DCSR_POS_DOPI)
#define BSP_OCTARAM_DCSR_MASK_ADLEN         (7   << BSP_OCTARAM_DCSR_POS_ADLEN)
#define BSP_OCTARAM_DCSR_MASK_DAOR          (1   << BSP_OCTARAM_DCSR_POS_DAOR)
#define BSP_OCTARAM_DCSR_MASK_CMDLEN        (7   << BSP_OCTARAM_DCSR_POS_CMDLEN)
#define BSP_OCTARAM_DCSR_MASK_ACDV          (1   << BSP_OCTARAM_DCSR_POS_ACDV)
#define BSP_OCTARAM_DCSR_MASK_DMLEN         (255 << BSP_OCTARAM_DCSR_POS_DMLEN)
#define BSP_OCTARAM_DCSR_MASK_DALEN         (255 << BSP_OCTARAM_DCSR_POS_DALEN)

// DCSR Register configuration
#define BSP_OCTARAM_DCSR_ACDA(n)            (((n) << BSP_OCTARAM_DCSR_POS_ACDA) & BSP_OCTARAM_DCSR_MASK_ACDA)
#define BSP_OCTARAM_DCSR_DOPI(n)            (((n) << BSP_OCTARAM_DCSR_POS_DOPI) & BSP_OCTARAM_DCSR_MASK_DOPI)
#define BSP_OCTARAM_DCSR_ADLEN(n)           (((n) << BSP_OCTARAM_DCSR_POS_ADLEN) & BSP_OCTARAM_DCSR_MASK_ADLEN)
#define BSP_OCTARAM_DCSR_DAOR(n)            (((n) << BSP_OCTARAM_DCSR_POS_DAOR) & BSP_OCTARAM_DCSR_MASK_DAOR)
#define BSP_OCTARAM_DCSR_CMDLEN(n)          (((n) << BSP_OCTARAM_DCSR_POS_CMDLEN) & BSP_OCTARAM_DCSR_MASK_CMDLEN)
#define BSP_OCTARAM_DCSR_ACDV(n)            (((n) << BSP_OCTARAM_DCSR_POS_ACDV) & BSP_OCTARAM_DCSR_MASK_ACDV)
#define BSP_OCTARAM_DCSR_DMLEN(n)           (((n) << BSP_OCTARAM_DCSR_POS_DMLEN) & BSP_OCTARAM_DCSR_MASK_DMLEN)
#define BSP_OCTARAM_DCSR_DALEN(n)           (((n) << BSP_OCTARAM_DCSR_POS_DALEN) & BSP_OCTARAM_DCSR_MASK_DALEN)

// DCSTR Register Bit position
#define BSP_OCTARAM_DCSTR_POS_DVSELLO       14
#define BSP_OCTARAM_DCSTR_POS_DVSELHI       11
#define BSP_OCTARAM_DCSTR_POS_DVSELCMD      8

// DCSTR Register Bit Mask
#define BSP_OCTARAM_DCSTR_MASK_DVSELLO      (3   << BSP_OCTARAM_DCSTR_POS_DVSELLO)
#define BSP_OCTARAM_DCSTR_MASK_DVSELHI      (7   << BSP_OCTARAM_DCSTR_POS_DVSELHI)
#define BSP_OCTARAM_DCSTR_MASK_DVSELCMD     (7   << BSP_OCTARAM_DCSTR_POS_DVSELCMD)

// DCSTR Register configuration
#define BSP_OCTARAM_DCSR_DVSELLO(n)         (((n) << BSP_OCTARAM_DCSTR_POS_DVSELLO) & BSP_OCTARAM_DCSTR_MASK_DVSELLO)
#define BSP_OCTARAM_DCSR_DVSELHI(n)         (((n) << BSP_OCTARAM_DCSTR_POS_DVSELHI) & BSP_OCTARAM_DCSTR_MASK_DVSELHI)
#define BSP_OCTARAM_DCSR_DVSELCMD(n)        (((n) << BSP_OCTARAM_DCSTR_POS_DVSELCMD) & BSP_OCTARAM_DCSTR_MASK_DVSELCMD)

// OctaRAM Device Mode Register Bit position
#define BSP_OCTARAM_DEVICE_POS_DPD_ENABLE   15
#define BSP_OCTARAM_DEVICE_POS_DRIVE_STR    12
#define BSP_OCTARAM_DEVICE_POS_RFSH_RATE    10
#define BSP_OCTARAM_DEVICE_POS_LAT_CODE     4
#define BSP_OCTARAM_DEVICE_POS_LAT_TYPE     3
#define BSP_OCTARAM_DEVICE_POS_BURST_TYPE   2
#define BSP_OCTARAM_DEVICE_POS_BURST_LEN    0

// OctaRAM Device Mode Register Bit mask
#define BSP_OCTARAM_DEVICE_MASK_DPD_ENABLE  (1   << BSP_OCTARAM_DEVICE_POS_DPD_ENABLE)
#define BSP_OCTARAM_DEVICE_MASK_DRIVE_STR   (3   << BSP_OCTARAM_DEVICE_POS_DRIVE_STR)
#define BSP_OCTARAM_DEVICE_MASK_RFSH_RATE   (3   << BSP_OCTARAM_DEVICE_POS_RFSH_RATE)
#define BSP_OCTARAM_DEVICE_MASK_LAT_CODE    (15  << BSP_OCTARAM_DEVICE_POS_LAT_CODE)
#define BSP_OCTARAM_DEVICE_MASK_LAT_TYPE    (1   << BSP_OCTARAM_DEVICE_POS_LAT_TYPE)
#define BSP_OCTARAM_DEVICE_MASK_BURST_TYPE  (1   << BSP_OCTARAM_DEVICE_POS_BURST_TYPE)
#define BSP_OCTARAM_DEVICE_MASK_BURST_LEN   (3   << BSP_OCTARAM_DEVICE_POS_BURST_LEN )

// OctaRAM Device Mode Register configuration
#define BSP_OCTARAM_DEVICE_DPD_ENABLE(n)   (((n) << BSP_OCTARAM_DEVICE_POS_DPD_ENABLE) & BSP_OCTARAM_DEVICE_MASK_DPD_ENABLE)
#define BSP_OCTARAM_DEVICE_DRIVE_STR(n)    (((n) << BSP_OCTARAM_DEVICE_POS_DRIVE_STR) & BSP_OCTARAM_DEVICE_MASK_DRIVE_STR)
#define BSP_OCTARAM_DEVICE_RFSH_RATE(n)    (((n) << BSP_OCTARAM_DEVICE_POS_RFSH_RATE) & BSP_OCTARAM_DEVICE_MASK_RFSH_RATE)
#define BSP_OCTARAM_DEVICE_LAT_CODE(n)     (((n) << BSP_OCTARAM_DEVICE_POS_LAT_CODE) & BSP_OCTARAM_DEVICE_MASK_LAT_CODE)
#define BSP_OCTARAM_DEVICE_LAT_TYPE(n)     (((n) << BSP_OCTARAM_DEVICE_POS_LAT_TYPE) & BSP_OCTARAM_DEVICE_MASK_LAT_TYPE)
#define BSP_OCTARAM_DEVICE_BURST_TYPE(n)   (((n) << BSP_OCTARAM_DEVICE_POS_BURST_TYPE) & BSP_OCTARAM_DEVICE_MASK_BURST_TYPE)
#define BSP_OCTARAM_DEVICE_BURST_LEN(n)    (((n) << BSP_OCTARAM_DEVICE_POS_BURST_LEN) & BSP_OCTARAM_DEVICE_MASK_BURST_LEN)

#endif
