/**
 * @file m13_rza2m_ek_pinout.c
 * @brief M13-RZ/A2M-EK pinout definition
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

// Dependencies
#include "m13_rza2m_ek_pinout.h"


/**
 * @brief LCD pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_lcd_pin_data[30] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_B_PIN_5, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA23
    {GPIO_PORT_B_PIN_4, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA22
    {GPIO_PORT_B_PIN_3, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA21
    {GPIO_PORT_B_PIN_2, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA20
    {GPIO_PORT_B_PIN_1, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA19
    {GPIO_PORT_B_PIN_0, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA18
    {GPIO_PORT_A_PIN_7, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA17
    {GPIO_PORT_A_PIN_6, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA16
    {GPIO_PORT_A_PIN_5, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA15
    {GPIO_PORT_A_PIN_4, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA14
    {GPIO_PORT_A_PIN_3, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA13
    {GPIO_PORT_A_PIN_2, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA12
    {GPIO_PORT_A_PIN_1, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA11
    {GPIO_PORT_A_PIN_0, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA10
    {GPIO_PORT_8_PIN_0, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA9
    {GPIO_PORT_F_PIN_0, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA8
    {GPIO_PORT_F_PIN_1, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA7
    {GPIO_PORT_F_PIN_2, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA6
    {GPIO_PORT_F_PIN_3, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA5
    {GPIO_PORT_F_PIN_4, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA4
    {GPIO_PORT_F_PIN_5, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA3
    {GPIO_PORT_F_PIN_6, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA2
    {GPIO_PORT_H_PIN_2, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA1
    {GPIO_PORT_F_PIN_7, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_DATA0
    {GPIO_PORT_C_PIN_4, {GPIO_FUNC_PERIPHERAL5, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_TCON3
    {GPIO_PORT_7_PIN_2, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_TCON2
    {GPIO_PORT_7_PIN_6, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_TCON1
    {GPIO_PORT_7_PIN_7, {GPIO_FUNC_OUT_LOW, GPIO_TINT_DISABLE, GPIO_CURRENT_8mA}},  // SIGNAL DISP
    {GPIO_PORT_J_PIN_6, {GPIO_FUNC_PERIPHERAL3, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}}, // LCD0_CLK
    {GPIO_PORT_J_PIN_7, {GPIO_FUNC_OUT_LOW, GPIO_TINT_DISABLE, GPIO_CURRENT_8mA}},     // LCD_BK_SHDN
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_lcd_pin_table[2] =
{
    &m13_rza2m_ek_lcd_pin_data[0],
    sizeof(m13_rza2m_ek_lcd_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief RIIC0 pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_riic0_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_D_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC0SCL
    {GPIO_PORT_D_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC0SDA
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_riic0_pin_table[2] =
{
    &m13_rza2m_ek_riic0_pin_data[0],
    sizeof(m13_rza2m_ek_riic0_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief RIIC1 pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_riic1_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_D_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC1SCL
    {GPIO_PORT_D_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC1SDA
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_riic1_pin_table[2] =
{
    &m13_rza2m_ek_riic1_pin_data[0],
    sizeof(m13_rza2m_ek_riic1_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief RIIC2 pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_riic2_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_D_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC2SCL
    {GPIO_PORT_D_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC2SDA
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_riic2_pin_table[2] =
{
    &m13_rza2m_ek_riic2_pin_data[0],
    sizeof(m13_rza2m_ek_riic2_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief RIIC3 pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_riic3_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_D_PIN_6, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC3SCL
    {GPIO_PORT_D_PIN_7, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},     // RIIC3SDA
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_riic3_pin_table[2] =
{
    &m13_rza2m_ek_riic3_pin_data[0],
    sizeof(m13_rza2m_ek_riic3_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief User button pinout table & structure
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_sw_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_E_PIN_0, {GPIO_FUNC_IN, GPIO_TINT_RESERVED, GPIO_CURRENT_4mA}},   // SW1
    {GPIO_PORT_5_PIN_4, {GPIO_FUNC_IN, GPIO_TINT_RESERVED, GPIO_CURRENT_4mA}},   // SW2
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_sw_pin_table[2] =
{
    &m13_rza2m_ek_sw_pin_data[0],
    sizeof(m13_rza2m_ek_sw_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief Led pinout table
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_led_pin_data[1] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_J_PIN_1, {GPIO_FUNC_OUT_HIGH, GPIO_TINT_DISABLE, GPIO_CURRENT_8mA}}, // USER LED
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_led_pin_table[2] =
{
    &m13_rza2m_ek_led_pin_data[0],
    sizeof(m13_rza2m_ek_led_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief Virtual Com pinout table
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_vcom_pin_data[2] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_4_PIN_6, {GPIO_FUNC_PERIPHERAL4, GPIO_TINT_DISABLE, GPIO_CURRENT_8mA}},       // TxD4
    {GPIO_PORT_4_PIN_5, {GPIO_FUNC_PERIPHERAL4, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}}, // RxD4
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_vcom_pin_table[2] =
{
    &m13_rza2m_ek_vcom_pin_data[0],
    sizeof(m13_rza2m_ek_vcom_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};


/**
 * @brief IRQ3 pinout table
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_irq1_pin_data[1] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
    {GPIO_PORT_5_PIN_5, {GPIO_FUNC_PERIPHERAL2, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}}, // IRQ1 for Touchscreen
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_irq1_pin_table[2] =
{
    &m13_rza2m_ek_irq1_pin_data[0],
    sizeof(m13_rza2m_ek_irq1_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief Ethernet pinout table
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_eth_pin_data[10] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description

	// ETHERNET IRQ
	{GPIO_PORT_3_PIN_0, {GPIO_FUNC_PERIPHERAL5, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}}, 	// IRQ3

	// ETHERNET CONFIG
	{GPIO_PORT_3_PIN_2, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},		// 	ET1_CRS
	{GPIO_PORT_3_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}},			// 	ET1_MDC
	{GPIO_PORT_3_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_4mA}},			// 	ET1_MDIO
	{GPIO_PORT_K_PIN_3, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},		// 	REF50CK1
	{GPIO_PORT_K_PIN_4, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},		// 	RMII1_RXD0
	{GPIO_PORT_3_PIN_5, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_RESERVED}},		// 	RMII1_RXD1
	{GPIO_PORT_K_PIN_0, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},			// 	RMII1_TXD_EN
	{GPIO_PORT_K_PIN_1, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},			// 	RMII1_TXD0
	{GPIO_PORT_K_PIN_2, {GPIO_FUNC_PERIPHERAL7, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},			// 	RMII1_TXD1
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_eth_pin_table[2] =
{
    &m13_rza2m_ek_eth_pin_data[0],
    sizeof(m13_rza2m_ek_eth_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};

/**
 * @brief Ethernet pinout table
 **/
const st_r_drv_gpio_sc_config_t m13_rza2m_ek_sdram_pin_data[38] =
{
    //{ <pin>           , {<function>           , <tint>           , <current>       }},	// Pin description
	 // SDRAM DATA CONFIG
	{GPIO_PORT_0_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D0
	{GPIO_PORT_0_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D1
	{GPIO_PORT_0_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D2
	{GPIO_PORT_0_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D3
	{GPIO_PORT_0_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D4
	{GPIO_PORT_0_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D5
	{GPIO_PORT_0_PIN_6, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D6
	{GPIO_PORT_1_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D7
	{GPIO_PORT_1_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D8
	{GPIO_PORT_1_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D9
	{GPIO_PORT_1_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D10
	{GPIO_PORT_1_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D11
	{GPIO_PORT_2_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D12
	{GPIO_PORT_2_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D13
	{GPIO_PORT_2_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D14
	{GPIO_PORT_2_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	D15

	// SDRAM ADDR CONFIG
	{GPIO_PORT_8_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A1
	{GPIO_PORT_8_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A2
	{GPIO_PORT_8_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A3
	{GPIO_PORT_8_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A4
	{GPIO_PORT_8_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A5
	{GPIO_PORT_8_PIN_6, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A6
	{GPIO_PORT_8_PIN_7, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A7
	{GPIO_PORT_9_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A8
	{GPIO_PORT_9_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A9
	{GPIO_PORT_9_PIN_2, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A10
	{GPIO_PORT_9_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A11
	{GPIO_PORT_9_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A12
	{GPIO_PORT_9_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A13
	{GPIO_PORT_9_PIN_6, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A14
	{GPIO_PORT_9_PIN_7, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	A15
	// SDRAM CTRL CONFIG
	{GPIO_PORT_6_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	CS3#
	{GPIO_PORT_6_PIN_7, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	DQML
	{GPIO_PORT_7_PIN_0, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	DQMU
	{GPIO_PORT_7_PIN_1, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	RD/WR#
	{GPIO_PORT_7_PIN_3, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	RAS#
	{GPIO_PORT_7_PIN_4, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	CAS#
	{GPIO_PORT_7_PIN_5, {GPIO_FUNC_PERIPHERAL1, GPIO_TINT_DISABLE, GPIO_CURRENT_12mA}},		// 	CKE
};

st_r_drv_gpio_pin_init_table_t m13_rza2m_ek_sdram_pin_table[2] =
{
    &m13_rza2m_ek_sdram_pin_data[0],
    sizeof(m13_rza2m_ek_sdram_pin_data) / sizeof(st_r_drv_gpio_sc_config_t),
};
