/**
 * @file m13_rza2m_ek_sdram.h
 * @brief M13-RZ/A2M-EK SDRAM BSP
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RZA2M_EK_SDRAM_H
#define _M13_RZA2M_EK_SDRAM_H

//Dependencies
#include <stdint.h>
#include <stdlib.h>
#include "m13_rza2m_ek_status.h"

//SDRAM base address (cache enabled)
//#define SDRAM_BASE_ADDR_NO_CACHE 0x0C000000
#define OCTARAM_BASE_ADDR_NO_CACHE 0x60000000

//SDRAM size, in bytes
#define SDRAM_SIZE (8 * 1024 * 1024)

//SDRAM data bus mask
#define SDRAM_DATA_BUS_MASK 0xFFFF
//SDRAM address bus mask
#define SDRAM_ADDR_BUS_MASK 0x7FFF

//CS3BCR register
#define BSC_CS3BCR_IWW_POS 28
#define BSC_CS3BCR_IWW_MASK (7 << BSC_CS3BCR_IWW_POS)
#define BSC_CS3BCR_IWW_VAL(n) (((n) << BSC_CS3BCR_IWW_POS) & BSC_CS3BCR_IWW_MASK)

#define BSC_CS3BCR_IWRWD_POS 25
#define BSC_CS3BCR_IWRWD_MASK (7 << BSC_CS3BCR_IWRWD_POS)
#define BSC_CS3BCR_IWRWD_VAL(n) (((n) << BSC_CS3BCR_IWRWD_POS) & BSC_CS3BCR_IWRWD_MASK)

#define BSC_CS3BCR_IWRWS_POS 22
#define BSC_CS3BCR_IWRWS_MASK (7 << BSC_CS3BCR_IWRWS_POS)
#define BSC_CS3BCR_IWRWS_VAL(n) (((n) << BSC_CS3BCR_IWRWS_POS) & BSC_CS3BCR_IWRWS_MASK)

#define BSC_CS3BCR_IWRRD_POS 19
#define BSC_CS3BCR_IWRRD_MASK (7 << BSC_CS3BCR_IWRRD_POS)
#define BSC_CS3BCR_IWRRD_VAL(n) (((n) << BSC_CS3BCR_IWRRD_POS) & BSC_CS3BCR_IWRRD_MASK)

#define BSC_CS3BCR_IWRRS_POS 16
#define BSC_CS3BCR_IWRRS_MASK (7 << BSC_CS3BCR_IWRRS_POS)
#define BSC_CS3BCR_IWRRS_VAL(n) (((n) << BSC_CS3BCR_IWRRS_POS) & BSC_CS3BCR_IWRRS_MASK)

#define BSC_CS3BCR_TYPE_POS 12
#define BSC_CS3BCR_TYPE_MASK (7 << BSC_CS3BCR_TYPE_POS)
#define BSC_CS3BCR_TYPE_VAL(n) (((n) << BSC_CS3BCR_TYPE_POS) & BSC_CS3BCR_TYPE_MASK)

#define BSC_CS3BCR_B11_POS 11
#define BSC_CS3BCR_B11_MASK (3 << BSC_CS3BCR_B11_POS)
#define BSC_CS3BCR_B11_VAL(n) (((n) << BSC_CS3BCR_B11_POS) & BSC_CS3BCR_B11_MASK)

#define BSC_CS3BCR_BSZ_POS 9
#define BSC_CS3BCR_BSZ_MASK (3 << BSC_CS3BCR_BSZ_POS)
#define BSC_CS3BCR_BSZ_VAL(n) (((n) << BSC_CS3BCR_BSZ_POS) & BSC_CS3BCR_BSZ_MASK)

//CS3WCR register
#define BSC_CS3WCR_WTRP_POS 13
#define BSC_CS3WCR_WTRP_MASK (1 << BSC_CS3WCR_WTRP_POS)
#define BSC_CS3WCR_WTRP_VAL(n) (((n) << BSC_CS3WCR_WTRP_POS) & BSC_CS3WCR_WTRP_MASK)

#define BSC_CS3WCR_WTRCD_POS 10
#define BSC_CS3WCR_WTRCD_MASK (3 << BSC_CS3WCR_WTRCD_POS)
#define BSC_CS3WCR_WTRCD_VAL(n) (((n) << BSC_CS3WCR_WTRCD_POS) & BSC_CS3WCR_WTRCD_MASK)

#define BSC_CS3WCR_A3CL_POS 7
#define BSC_CS3WCR_A3CL_MASK (3 << BSC_CS3WCR_A3CL_POS)
#define BSC_CS3WCR_A3CL_VAL(n) (((n) << BSC_CS3WCR_A3CL_POS) & BSC_CS3WCR_A3CL_MASK)

#define BSC_CS3WCR_TRWL_POS 3
#define BSC_CS3WCR_TRWL_MASK (3 << BSC_CS3WCR_TRWL_POS)
#define BSC_CS3WCR_TRWL_VAL(n) (((n) << BSC_CS3WCR_TRWL_POS) & BSC_CS3WCR_TRWL_MASK)

#define BSC_CS3WCR_WTRC_POS 0
#define BSC_CS3WCR_WTRC_MASK (3 << BSC_CS3WCR_WTRC_POS)
#define BSC_CS3WCR_WTRC_VAL(n) (((n) << BSC_CS3WCR_WTRC_POS) & BSC_CS3WCR_WTRC_MASK)

//SDCR register
#define BSC_SDCR_A2ROW_POS 19
#define BSC_SDCR_A2ROW_MASK (3 << BSC_SDCR_A2ROW_POS)
#define BSC_SDCR_A2ROW_VAL(n) (((n) << BSC_SDCR_A2ROW_POS) & BSC_SDCR_A2ROW_MASK)

#define BSC_SDCR_A2COL_POS 16
#define BSC_SDCR_A2COL_MASK (3 << BSC_SDCR_A2COL_POS)
#define BSC_SDCR_A2COL_VAL(n) (((n) << BSC_SDCR_A2COL_POS) & BSC_SDCR_A2COL_MASK)

#define BSC_SDCR_DEEP_POS 13
#define BSC_SDCR_DEEP_MASK (1 << BSC_SDCR_DEEP_POS)
#define BSC_SDCR_DEEP_VAL(n) (((n) << BSC_SDCR_DEEP_POS) & BSC_SDCR_DEEP_MASK)

#define BSC_SDCR_RFSH_POS 11
#define BSC_SDCR_RFSH_MASK (1 << BSC_SDCR_RFSH_POS)
#define BSC_SDCR_RFSH_VAL(n) (((n) << BSC_SDCR_RFSH_POS) & BSC_SDCR_RFSH_MASK)

#define BSC_SDCR_RMODE_POS 10
#define BSC_SDCR_RMODE_MASK (1 << BSC_SDCR_RMODE_POS)
#define BSC_SDCR_RMODE_VAL(n) (((n) << BSC_SDCR_RMODE_POS) & BSC_SDCR_RMODE_MASK)

#define BSC_SDCR_PDOWN_POS 9
#define BSC_SDCR_PDOWN_MASK (1 << BSC_SDCR_PDOWN_POS)
#define BSC_SDCR_PDOWN_VAL(n) (((n) << BSC_SDCR_PDOWN_POS) & BSC_SDCR_PDOWN_MASK)

#define BSC_SDCR_BACTV_POS 8
#define BSC_SDCR_BACTV_MASK (1 << BSC_SDCR_BACTV_POS)
#define BSC_SDCR_BACTV_VAL(n) (((n) << BSC_SDCR_BACTV_POS) & BSC_SDCR_BACTV_MASK)

#define BSC_SDCR_A3ROW_POS 3
#define BSC_SDCR_A3ROW_MASK (3 << BSC_SDCR_A3ROW_POS)
#define BSC_SDCR_A3ROW_VAL(n) (((n) << BSC_SDCR_A3ROW_POS) & BSC_SDCR_A3ROW_MASK)

#define BSC_SDCR_A3COL_POS 0
#define BSC_SDCR_A3COL_MASK (3 << BSC_SDCR_A3COL_POS)
#define BSC_SDCR_A3COL_VAL(n) (((n) << BSC_SDCR_A3COL_POS) & BSC_SDCR_A3COL_MASK)

//RTCOR register
#define BSC_RTCOR_KEY_POS 16
#define BSC_RTCOR_KEY_MASK (65535 << BSC_RTCOR_KEY_POS)
#define BSC_RTCOR_KEY_VAL(n) (((n) << BSC_RTCOR_KEY_POS) & BSC_RTCOR_KEY_MASK)

//RTCSR register
#define BSC_RTCSR_KEY_POS 16
#define BSC_RTCSR_KEY_MASK (65535 << BSC_RTCSR_KEY_POS)
#define BSC_RTCSR_KEY_VAL(n) (((n) << BSC_RTCSR_KEY_POS) & BSC_RTCSR_KEY_MASK)

#define BSC_RTCSR_RTCSR_POS 0
#define BSC_RTCSR_RTCSR_MASK (65535 << BSC_RTCSR_RTCSR_POS)
#define BSC_RTCSR_RTCSR_VAL(n) (((n) << BSC_RTCSR_RTCSR_POS) & BSC_RTCSR_RTCSR_MASK)

#define BSC_RTCSR_CMF_POS 7
#define BSC_RTCSR_CMF_MASK (1 << BSC_RTCSR_CMF_POS)
#define BSC_RTCSR_CMF_VAL(n) (((n) << BSC_RTCSR_CMF_POS) & BSC_RTCSR_CMF_MASK)

#define BSC_RTCSR_CMIE_POS 6
#define BSC_RTCSR_CMIE_MASK (1 << BSC_RTCSR_CMIE_POS)
#define BSC_RTCSR_CMIE_VAL(n) (((n) << BSC_RTCSR_CMIE_POS) & BSC_RTCSR_CMIE_MASK)

#define BSC_RTCSR_CKS_POS 3
#define BSC_RTCSR_CKS_MASK (7 << BSC_RTCSR_CKS_POS)
#define BSC_RTCSR_CKS_VAL(n) (((n) << BSC_RTCSR_CKS_POS) & BSC_RTCSR_CKS_MASK)

#define BSC_RTCSR_RRC_POS 0
#define BSC_RTCSR_RRC_MASK (7 << BSC_RTCSR_RRC_POS)
#define BSC_RTCSR_RRC_VAL(n) (((n) << BSC_RTCSR_RRC_POS) & BSC_RTCSR_RRC_MASK)

//SDRAM mode register
#define SDRAM_MODE_CS3 (*(volatile uint16_t *)(0x1F002040)) // 0x3FFFE040

//SDRAM related functions
bsp_status_t bsp_sdram_io_init(void);
void bsp_sdram_init(void);

#endif
