/**
 * @file m13_rza2m_ek_vcom.c
 * @brief M13-RZ/A2M-EK Virtual COM definition
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#include "m13_rza2m_ek_vcom.h"

// Private global variables and functions


// Macro definitions


/**
 * @name       bsp_vcom_pin_init
 * @brief      Configure M13-RZA2M-EK virtual COM pins
 * @param[in]  None
 * @return     None
 */
bsp_status_t bsp_vcom_init(void)
{
	int_t gpio_handle, err;
	
    gpio_handle = direct_open("gpio", 0);
    if(gpio_handle < 0)
    {
		return BSP_STATUS_FAILURE;
    }

    err = direct_control(gpio_handle, CTL_GPIO_INIT_BY_TABLE, (st_r_drv_gpio_pin_init_table_t *) &m13_rza2m_ek_vcom_pin_table);

    if(err < 0)
    {
        return BSP_STATUS_PIN_CONFIG_FAILURE;
    }
	
	return BSP_STATUS_OK;
}

void debugDisplayArray(FILE *stream,
   const char_t *prepend, const void *data, size_t length)
{
   uint_t i;

   for(i = 0; i < length; i++)
   {
      //Beginning of a new line?
      if((i % 16) == 0)
    	  fprintf(stream, "%s", prepend);//fprintf(stream, "%s", prepend);

      //Display current data byte
      fprintf(stream, "%02X" " ", *((uint8_t *) data + i)); //fprintf(stream, "%02" PRIX8 " ", *((uint8_t *) data + i));
      //End of current line?
      if((i % 16) == 15 || i == (length - 1))
    	  fprintf(stream, "\r\n"); //fprintf(stream, "\r\n");
   }
}
