/**
 * @file m13_rza2m_ek_vcom.h
 * @brief M13-RZ/A2M-EK Virtual COM definition
 *
 * @section License
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Copyright (C) 2020-2021 M13DESIGN SARL. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author M13DESIGN SARL (www.m13design.fr)
 * @version 1.0.0
 **/

#ifndef _M13_RZA2M_EK_VCOM_H
#define _M13_RZA2M_EK_VCOM_H

// Dependencies
#include "FreeRTOS.h"
#include "task.h"
#include "r_gpio_drv_api.h"
#include "m13_rza2m_ek_pinout.h"
#include "m13_rza2m_ek_status.h"

// Trace level definitions
#define TRACE_LEVEL_OFF      0
#define TRACE_LEVEL_INFO     1
#define TRACE_LEVEL_DEBUG    2

// Default trace level
#ifndef TRACE_LEVEL
   #define TRACE_LEVEL TRACE_LEVEL_INFO
#endif



// Macro definitions
#define _PRINTF(...) fprintf(stderr, __VA_ARGS__)
#define _PRINTF_ARRAY(p, a, n) debugDisplayArray(stderr, p, a, n)


#if (TRACE_LEVEL >= TRACE_LEVEL_INFO)
#define PRINTF_INFO(...)         _PRINTF(__VA_ARGS__)
#define PRINTF_ARRAY(p, a, n)    _PRINTF_ARRAY(p, a, n)
#else
   #define PRINTF_INFO(...)
   #define PRINTF_ARRAY(p, a, n)
#endif

#if (TRACE_LEVEL >= TRACE_LEVEL_DEBUG)
   #define PRINTF_DEBUG(...)     _PRINTF(__VA_ARGS__)
   #define PRINTF_ARRAY_DEBUG(p, a, n)    _PRINTF_ARRAY(p, a, n)
#else
   #define PRINTF_DEBUG(...)
   #define PRINTF_ARRAY_DEBUG(p, a, n)
#endif


// BSP Functions
bsp_status_t bsp_vcom_init(void);

// Other functions
void debugDisplayArray(FILE *stream,
   const char_t *prepend, const void *data, size_t length);

#endif

